# Add the step to overlap TFs with the promoter regions
# 
# Author: Judith Zaugg
###############################################################################
# TFs=TFs_all[,c('first','last','seqName','regulator','id','tx_ID','TSS')]
# TFs$tx_ID=gsub('ST','',TFs$tx_ID)

TFenrichmentPlots=function(
		TFs=NULL,
		GlobRegNo=50,
		clusterTab=NULL,
		annoTab=NULL,
		projectfolder=getwd(),
		category='YPD.NFR.cat',
		significant=0.05,
		xlab='',ylab=''){
	
	require(lattice)
	require(latticeExtra)
	if(is.null(TFs)){
		data(TFs)
		cat('loaded TFs from package AnalyzeGenomicPipeline\n')
	}
# define global regulators as TFs that have more than 50 binding sites
	GlobReg=names(table(TFs$regulator))[order(table(TFs$regulator),decreasing=TRUE)]
	GlobReg=names(table(TFs$regulator))[table(TFs$regulator)>GlobRegNo]
	cat(length(GlobReg),'global regulators with more than',GlobRegNo,'targets\n')
# set the variable that controls whether a category is given to test extra
	Cat_specific=ifelse(!is.null(category),TRUE,FALSE)
# load clusterTab if not provided
	annoTab=loadAnnoTab(projectfolder)
	clusterTab=loadClusterTab(projectfolder,clusterTab,TRUE)
	
	ft_L=list()
	if(Cat_specific){
		cats=c(unique(annoTab[[category]]),'all')
# add the category that is going to be analyzed separately to the TFs
		TFs$category=annoTab[[category]][match(TFs$tx_ID,annoTab$id)]
		clusterTab$cat=annoTab[[category]][match(clusterTab$id,annoTab$id)]
	} else{clusterTab$cat=1;cats=1}
# calculate the enrichment for each of the global regulators
	for(j in 1:length(GlobReg)){
		reg=GlobReg[j]
		clusterTab$Reg=ifelse(clusterTab$id%in%TFs$tx_ID[TFs$regulator==reg],'present','absent')
		for(k in cats){
			if(k=='all'){
				kk=unique(clusterTab$cat)
			}
			else
				kk=k
			for(i in unique(clusterTab$clustering)){
				ct=clusterTab[clusterTab$cat%in%kk,]
				if(with(ct,min(dim(table(clustering==i,Reg)))>1 )){
					ft=with(clusterTab[clusterTab$cat%in%kk,],
							fisher.test(clustering==i,Reg))
					ft_L[[paste(k,i,j)]]=c('p.value'=ft$p.value,'odds'=ft$estimate,'reg'=reg,'cluster'=i,'cat'=k)
				}
			}
		}
		cat(j,'\r')
	}
# collect all information in one data.frame called ft
	ft=as.data.frame(do.call(rbind,ft_L))
# calculate log ratios and adjusted pvalues	and set non-significant values to zero
	cat('calculate log ratio and adjust p-values\n')
	ft$logRatio=log(as.numeric(as.character(ft$odds)))
	ft$p.adjust=p.adjust(as.numeric(as.character(ft$p.value)),method='BH')
	ft$logRatioCor=ifelse(ft$p.adjust<significant,ft$logRatio,NA) # set logRatioC to NA if it is not significant
	if(Cat_specific){
		ft$cat_clust=paste(ft$cluster,ft$cat)
	} else{ft$cat_clust=paste(ft$cluster)}
	TF_enrichment=ft
	save(TF_enrichment,file=paste(projectfolder,'TF_enrichment.rda',sep='/'))
	cat('TF_enrichment saved in',paste(projectfolder,'TF_enrichment.rda',sep=''),'\n')
# make the enrichment plot	
	LL=split(ft$logRatioCor,as.character(ft$cat_clust))
	mat=do.call(rbind,LL)
	colnames(mat)=unique(ft$reg)
	mat=mat[,!is.na(colMeans(mat,na.rm=T))]
	mat=mat[,is.finite(colMeans(mat,na.rm=T))]
	for(j in 1:nrow(mat)){
		mat[j,is.na(mat[j,])]=0
	}	
	dd.row <- as.dendrogram(hclust(dist((mat))))
	row.ord <- order.dendrogram(dd.row)
	dd.col <- as.dendrogram(hclust(dist(t(mat))))
	col.ord <- order.dendrogram(dd.col)
	colores=colorRampPalette(c('red','white','blue'))
# make the levelplot
	cat('make the plot\n')
	lp=levelplot((mat[,col.ord]),
			aspect='fill',
			scales=list(y=list(cex=0.7),x=list(cex=0.7,rot=70)),
			at=seq(-max(abs(mat)),max(abs(mat)),0.1),
			col.regions=colores,
			cluster=unique(ft$cluster),
			cat_clust=unique(ft$cat_clust),
			colorkey=list(space='left'),
			panel=panFunTFs
	)
	plotfile=paste(projectfolder,'plots/TF_heatmap.png',sep='/')
	png(plotfile,width=(length(unique(ft$cat_clust))*50+50),height=ncol(mat)*50+50)
	trellis.par.set(Tset)
	plot(update(lp,xlab=xlab,ylab=ylab))
	dev.off()
	return(plotfile)
}

panFunTFs=function(...){
	panel.levelplot(...)
	ll1=length(unique(ft$cat_clust))
	ll2=length(unique(ft$cluster))
	panel.xyplot(seq(ll1/ll2,ll1,ll1/ll2)+0.5,nrow(ft),type='h',lty=1,lwd=1,col='grey' )
}