# This script contains the functions to plot the cluster profiles
#
# TO DO: implement category feature in plot!
# Author: Judith Zaugg
###############################################################################

plotProfiles=function(
		clusterTab=NULL,
		mat,
		label=NULL,
		category=NULL,
		projectfolder=getwd(),
		xlab='',ylab='',main='', 
		plotfolder=NULL,
		plot.settings=Tset,
		device='png'){
	require(lattice)
	require(latticeExtra)
	
# check the inputs
# projectfolder
	projectfolder=gsub('/$','',projectfolder)
	cat(createFolderIfNotExist(projectfolder))
	if(is.null(plotfolder)){
		plotfolder=paste(projectfolder,'plots',sep='/')
	} 
	plotfolder=gsub('/$','',plotfolder)
	cat(createFolderIfNotExist(plotfolder))
	# load clusterTab if not supplied
	clusterTab=loadClusterTab(projectfolder,clusterTab,force.save=FALSE)
	
	if(class(clusterTab)=='GRanges')
		clusterTab=as.data.frame(elementMetadata(clusterTab))
# clusterTab
	stopifnot(!is.null(clusterTab$clustering),!is.null(clusterTab$silinfo),!is.null(clusterTab$id))
# select clusterTab that are present in mat and sort according to clustering and silinfo
	clusterTab=clusterTab[clusterTab$id%in%c(rownames(mat)),]
	stopifnot(dim(clusterTab)[1]!=0)
	if(!is.null(category)){
		clusterTab$cat=factor(clusterTab[[category]])
	}else{
		clusterTab$cat=0
	}
	clusterTab=clusterTab[order(clusterTab$clustering,clusterTab$silinfo,clusterTab$cat),]
	
# mat: sort according to the clusterTab$id
	mat=mat[match(clusterTab$id,rownames(mat)),]
	if(length(grep('_',colnames(mat)))>0){
		labels=only.text(colnames(mat))
		positions=only.numeric(colnames(mat))
		cat('found',length(unique(labels)),'different factors to be plotted:',unique(labels),'\n')
	}else{
		stopifnot(!is.null(label))
		labels=label
		positions=as.numeric(colnames(mat))
		colnames(mat)=paste(colnames(mat),labels,sep='_')
		cat('found only one facter to plot:',label,'\n')
	}
	
# special features used in the plotting function	
	clust=clusterTab$clustering
	no_clust=length(unique(clust))# number of clusters -> needed in axis labels
	cat=clusterTab$cat
	no_cat=length(unique(cat))# number of clusters -> needed in axis labels
	
	lp_L=list()# this is to collect all the plots 
	lims_L=list()
	for(ll in unique(labels)){
		mat_L=split(as.data.frame(mat[,only.text(colnames(mat))==ll]),clust)
		plot_df=data.frame(
				value=c(do.call('c',lapply(mat_L,colMeans,na.rm=T)),
						do.call('c',lapply(mat_L,rowQuantiles,0.75)),
						do.call('c',lapply(mat_L,rowQuantiles,0.25)),
						do.call('c',lapply(mat_L,rowQuantiles,0.5))),
				cluster=rep(names(mat_L),rep(ncol(mat_L[[1]]),length(mat_L))),
				position=only.numeric(colnames(mat_L[[1]])))
		plot_df$type=factor(rep(c('mean','75','25','median'),rep(nrow(plot_df)/4,4)),levels=c('mean','median','25','75'))
		plot_df$factor=ll
		cat('plotted',ll,'\n')
		lp_L[[ll]]=plot_df
		lims_L[[ll]]=c(min(plot_df$value,na.rm=T),max(plot_df$value,na.rm=T))
	}
	plotDF=as.data.frame(do.call(rbind,lp_L))
	plotDF$factor=factor(plotDF$factor,levels=names(lp_L))
	if(class(clusterTab$clustering)=='factor')
		plotDF$cluster=factor(plotDF$cluster,levels=levels(clusterTab$clustering))
	lims=rep(lims_L,rep(length(unique(clust)),length(lp_L)))
	L <- vector('list', length(lp_L)*length(unique(clust)))
	L[c(seq(1,length(L),length(unique(clust))))]<-NA
# plot the profiles
	xy=xyplot(value~position|cluster*factor,groups=type,data=plotDF,type='l',
			panel=function(...){
				panel.xyplot(...)
				panel.abline(0,0,col='black',lwd=1)
			})
	
# convert the plot so that only the outer stripes are shown
	xy=useOuterStrips(update(xy,
					xlab=xlab,ylab=ylab,main=main,
					scales=list(alternating=FALSE,
							y=list(relation='free',
									limits=lims, at=L)),
					between=list(x = -1.75)))
# set the plotting parameters, Tset must be inputted
	plot.settings$superpose.line$lty=c(1,1,2,2)
	plot.settings$superpose.line$col=c('grey','black','black','black')
	len=length(unique(clust))
	no=length(unique(labels))
# filename
	if(device=='png'){
		profilefile=paste(plotfolder,'/',label,'profiles.png',sep='')
		png(profilefile,width=(len*70+20),height=(no*70+20))
	} else{	
		profilefile=paste(plotfolder,'/',label,'profiles.pdf',sep='')
		pdf(profilefile,width=(len*70+10)/70,height=(no*70+20)/70)
	}
	trellis.par.set(plot.settings)
	plot(xy)
	dev.off()
	cat('profile plots have been plotted in',profilefile,'\n')
	return(profilefile)
}
rowQuantiles=function(x,p=0.5)apply(x,MARGIN=2,quantile,p,na.rm=T)
